from dune.perftool.generation import (cached,
                                      domain,
                                      iname,
                                      instruction,
                                      symbol,
                                      temporary_variable,
                                      )
from dune.perftool.pdelab.geometry import (name_geometry,
                                           )

@iname
def quadrature_iname():
    domain("q", "q_n")
    return "q"


def quadrature_preamble(code, **kw):
    return instruction(inames=quadrature_iname(), code=code, **kw)


@symbol
def name_quadrature_point():
    return "qp"


@symbol
def name_quadrature_position():
    qp = name_quadrature_point()
    return "{}.position()".format(qp)


@symbol
def name_quadrature_weight():
    qp = name_quadrature_point()
    return "{}.weight()".format(qp)


def define_quadrature_factor(fac):
    weight = name_quadrature_weight()
    geo = name_geometry()
    pos = name_quadrature_position()
    code = "{} = {}*{}.integrationElement({});".format(fac,
                                                       weight,
                                                       geo,
                                                       pos,
                                                       )
    return quadrature_preamble(code, assignees=fac)


@symbol
def name_factor():
    temporary_variable("fac", shape=())
    define_quadrature_factor("fac")
    return "fac"


@symbol
def name_order():
    # TODO how to determine integration order in a generic fashion here?
    # Or maybe just read it from the ini file? Or define some mapping from the
    # finite elements?
    return "2"


@cached
def quadrature_loop_statement():
    qp = name_quadrature_point()
    geo = name_geometry()
    order = name_order()
    return "const auto& {} : quadratureRule({}, {})".format(qp,
                                                            geo,
                                                            order,
                                                            )
